# frozen_string_literal: true

require 'bundler'
Bundler.require

# TODO: Переделать.
class Artabl
  require_relative 'artabl/cell'
  require_relative 'artabl/collection'
  require_relative 'artabl/version'

  Error = Class.new(StandardError)
  DataNotHashError = Class.new(Error)

  def initialize(data)
    @data = data
  end

  attr_reader :data

  def self.convert(data)
    new(data).convert
  end

  def convert
    validate!
    size_columns
  end

  private

  def norm_data
    data.is_a?(Array) ? data : [data]
  end

  def validate!
    norm_data.each { |row| raise DataNotHashError unless row.is_a?(Hash) }
  end

  def headers
    norm_data.map(&:keys).flatten.uniq
  end

  def arr_table
    result_rows = []
    result_rows << headers

    norm_data.each do |data_row|
      result_row = headers.map do |h|
        data_row[h]
      end

      result_rows << result_row
    end

    result_rows
  end

  def size_columns
    sizes = []

    arr_table.each do |row|
      row.each_with_index do |col, idx|
        sizes[idx] ||= 0

        next if col.nil?

        sizes[idx] = col.size if col.size > sizes[idx]
      end
    end

    sizes
  end
end
